function W = computeWeights(maxError, learningRate, nodeCount, input,      ...
                            desired, logisticWeight)
% This program uses back-propagation to train a neural network to perform the 
% XOR operation
%
% maxError is the max mean error vector length a trained network may have
% learningRate is the "strength" of each weight update.
% nodeCount is an array containing the number of nodes per layer (including inputs)
% input is a cell array of training data input components
% desired is a cell array of training data desired components
% logisticWeight is the weight of the logistic activation functions
%
% this program returns the weight of every connection

% Set up some constants
global W
rand('state',0);
randn('state',0);



% Make sure our inputs and desired outputs are the same size
if length(input) ~= length(desired);
        error('The input vectors must all be the same size.');
end;

% Set up the weights
% W{} contains a set of matrices, one for each layer.  The matrix for 
% weight-layer k contains elements W{k}(j,i) connecting the output of 
% neuron i to the input of neuron j.
for layer=1:length(nodeCount)-1;
    W{layer} = 0.1 * randn(nodeCount(layer+1), nodeCount(layer));
end;

% Iterate over the set of training data, updating the weights each time
iteration = 0;
while 1;
    
    drawnow;
    % Randomize the order of our input data and our desired response.
    order = randperm(length(input));
    input = input(order,:);
    desired = desired(order,:);
    
    % Present each element in the training set once
    meanError = 0;
    for trainingElement = 1:length(input);
        
        % Get this input vector and desired output vector
        x = input(trainingElement,:);
        d = desired(trainingElement,:);
        
        % Find the output for this input
        y = computeOutputs(nodeCount, x, W, logisticWeight);
        
        % Keep track of the mean error vector magnitude
        error = d-y{length(nodeCount)};
        meanError = meanError + sqrt(dot(error, error));

		% Update the weights by back propagating
		for layer=length(nodeCount):-1:2;
            
            % Compute the local gradient for each node
            if layer == length(nodeCount);
                delta{layer-1} = ...
                    (d - y{layer}) ...
                    .* logisticWeight .* y{layer} .* ( 1 - y{layer} );
            else
                delta{layer-1} = ...
                    dot(repmat(delta{layer}', 1, size(W{layer}, 2)), W{layer})...
                    .* logisticWeight .* y{layer} .* ( 1 - y{layer} );
            end;
            
		    % Update the weights
            W{layer-1} = W{layer-1} + learningRate * reshape( ...
                                kron(y{layer-1}, delta{layer-1}), ...
                                nodeCount(layer), ...
                                nodeCount(layer-1) );
        end;
		
    end;    % the iteration over training elements
    
    % Find the mean error to determine whether we're done iterating
    meanError = meanError / length(input);
    iteration = iteration + 1;
    errorHistory(iteration) = meanError;
    if mod(iteration, 25) == 0
        disp(sprintf('Iteration: %d, mean error magnitude: %e', iteration, meanError))
        lastIteration = iteration - 25;
        if lastIteration < 1;
            lastIteration = 1;
        end;
        line([lastIteration, iteration], [errorHistory(lastIteration) meanError]); hold on;
    end;
    if meanError < maxError;
        break;
    end;
    
end;        % all iterations

disp(sprintf('Iterations: %d', iteration))
hold off;
return